require File.dirname(__FILE__) + '/probabilities.rb'

class Classifier
  def initialize
    @words_in_categories = Hash.new(0)
    @categories = Hash.new(0)
  end

  def learn( category, text )
    word_frequencies( text ).each do |word, count|
      @categories[ category ] += count
      @words_in_categories[ compound(category, word) ] += count
    end
  end

  def distribution( text )
    return measure( text ).normalised
  end

  def classify( text )
    return measure( text ).likely { |score, highest| score > 0.5 * highest }
  end

  private
  def word_frequencies( text )
    frequencies = Hash.new(0)
    text.downcase.split( /\s+/ ).each do |word|
      frequencies[ word ] += 1
    end
    return frequencies
  end
  
  def measure( text )
    probabilities = ProbabilityVector.new
    @categories.each do |category, category_count|
      word_frequencies( text ).each do |word, word_count|
        if @words_in_categories.has_key?( compound( category, word ) )
          probabilities.multiply( category,
              1.0 * @words_in_categories[ compound( category, word ) ] *
              word_count / category_count )
        else
          probabilities.multiply( category, 0.1 / category_count )
        end
      end
    end
    return probabilities
  end
  
  def compound( category, word )
    category + ' ' + word
  end
end
